/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"

void etherxs_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);
    int oldtype = eh->type;

    eh->type = EHANDLER_ERROR;
    if(ts->flag_aux1 && oldtype != EHANDLER_ERROR)
    {
        log_warn("libetherx","Connection to etherxd failed, attempting to reconnect...");
        ehandler_connect(eh);
    }else{
        pth_cond_notify(&(eh->cond),FALSE);
    }
}

void etherxs_receive(thread t, xmlnode x, int flag)
{
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);
    epacket ep;

    switch(flag)
    {
        case XMLSTREAM_ROOT:
            log_debug(ZONE,"got root tag from etherx!");
            ts->flag_aux1 = 1;
            eh->type = EHANDLER_SOCKET; /* make sure we re-set our type if it was ERROR */
            pth_cond_notify(&(eh->cond),FALSE); /* let the connector know we're cool */
            /* ignore the root tag content, transports don't use it */
            break;
        case XMLSTREAM_BOUNCE:
            ep = epacket_new(xmlnode_dup(x), NULL, NULL, eh->namespace);
            log_error("libetherx","bouncing data intended for etherx");
            epacket_error(ep,"delivery failure");
            (*eh->h)(ep);
            break;
        case XMLSTREAM_NODE:
            log_debug(ZONE,"got node from etherx!");
            if(j_strcmp(xmlnode_get_name(x),"stream:error") == 0)
            {
                log_error("libetherx","connection to etherxd failed: %s",xmlnode_get_data(x));
                raise(SIGTERM);
                break;
            }
            ep = epacket_new(xmlnode_dup(x), eh->host, NULL, eh->namespace);
            (*eh->h)(ep);
            break;
        case XMLSTREAM_INIT:
            pth_cleanup_push(etherxs_cleanup, (void *)t);
            break;
    }
}

void etherxs_connect(ehandler eh)
{
    tstream ts;
    xmlnode x;

    log_debug(ZONE,"attempting socket connect to %s for transport %s",etherx__host,eh->host);

    eh->t = xmlstream_connect(5269, etherx__host, (xmlstream_onNode)etherxs_receive);
    ts = (tstream)(eh->t->data);
    ts->data_aux1 = (void *)eh; /* so that _etherx_receive can access the handler info */
    x = xmlstream_header(eh->namespace, etherx__host, eh->host);
    xmlnode_put_attrib(x, "xmlns:etherx", ETHERX_NAMESPACE);
    xmlnode_put_attrib(x, "etherx:secret", etherx__secret);
    xmlstream_write_header(eh->t, x);
}

