/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <etherx.h>

unsigned int etherx_stack_default = 16*1024;
int etherx_debug_flag = 0;

int log_device = LOGNONE;
thread log_device_file = NULL;

int log_init(int log2, char *arg)
{
    int fd = 0;

    /* EPIPE is easier to handle than a signal */
    signal(SIGPIPE, SIG_IGN);

    /* init pth */
    pth_init();	

    if(etherx_debug_flag)
        log2 = LOGSTDERR;

    switch(log2)
    {
        case LOGFILE:
            fd = open(arg, O_WRONLY | O_CREAT | O_APPEND, 0600);
            if(fd > 0)
                log_device_file = tstream_new(fd, "_log", NULL, NULL, NULL);
            else
                log2 = LOGSTDERR;
            break;
        case LOGSYSLOG:
            if(arg == NULL)
                openlog("etherx", LOG_PID, LOG_DAEMON);
            else
                openlog(arg, LOG_PID, LOG_DAEMON);
            break;
    }
    log_device = log2;

    log_debug(ZONE,"log_init");

    return 1;
}

void etherx_debug()
{
    etherx_debug_flag = 1;
    log_device = LOGSTDERR;
}

void _log_debug(char *zone, const char *msgfmt, ...)
{
	va_list ap;
	static char loghdr[LOGSIZE_HDR];
	static char logmsg[LOGSIZE_TAIL];
	static int size;

	/* XXX: We may want to check the sizes eventually */
	size = snprintf(loghdr, LOGSIZE_HDR, "%s debug/%s %s\n", 
					create_log_timestamp(),zone, msgfmt);

	va_start(ap, msgfmt);
	size = vsnprintf(logmsg, LOGSIZE_TAIL, loghdr, ap);
    
	_log(logmsg);

	return;
}

char *zonestr(char *file, int line)
{
    static char buff[64];
    char *dot, *f;

    f = strdup(file);
    dot = strstr(f,".");
    if(dot != NULL)
        *dot = '\0';

    snprintf(buff,63,"%s:%d",f,line);

    free(f);
    return buff;
}

char *create_log_timestamp(void)
{
	time_t t;
	int sz;
	char *tmp_str;

	t = time(NULL);

	if(t == (time_t)-1)
		return NULL; 
	
	tmp_str = ctime(&t);
	sz = strlen(tmp_str);
	/* chop off the \n */
	tmp_str[sz-1]=' ';

	return tmp_str;
}

void log_warn(char *zone, const char *msgfmt, ...)
{
	va_list ap;
	static char loghdr[LOGSIZE_HDR];
	static char logmsg[LOGSIZE_HDR];
	static int size;

	/* XXX: We may want to check the sizes eventually */
	size = snprintf(loghdr, LOGSIZE_HDR, "%s warn/%s %s\n",
					create_log_timestamp(), zone, msgfmt);

	va_start(ap, msgfmt);
	size = vsnprintf(logmsg, LOGSIZE_TAIL, loghdr, ap);
    
	_log(logmsg);

	return;
}

void log_error(char *zone, const char *msgfmt, ...)
{
	va_list ap;
	static char loghdr[LOGSIZE_HDR];
	static char logmsg[LOGSIZE_HDR];
	static int size;

	/* XXX: We may want to check the sizes eventually */
	size = snprintf(loghdr, LOGSIZE_HDR, "%s error/%s %s\n", 
					create_log_timestamp(), zone, msgfmt);

	va_start(ap, msgfmt);
	size = vsnprintf(logmsg, LOGSIZE_TAIL, loghdr, ap);
    
	_log(logmsg);

	return;
}

void _log(const char *logmsg)
{
    switch(log_device)
    {
        case LOGNONE:
            break;
        case LOGSTDERR:
            fprintf(stderr,"%s", logmsg);
            break;
        case LOGSYSLOG:
            syslog(LOG_NOTICE, logmsg);
            break;
        case LOGFILE:
            tstream_write(log_device_file, (char *)logmsg, -1);
            break;
    }
}

