/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <etherx.h>

thread xmlstream_connect(int port, char *host, xmlstream_onNode evt)
{
    log_debug(ZONE,"xmlstream_connect");
    return tstream_connect(port, host, _xmlstream_main, _xmlstream_error, evt);
}

thread xmlstream_new(int fd, char *host, xmlstream_onNode evt)
{
    log_debug(ZONE,"xmlstream_new");
    return tstream_new(fd, host, _xmlstream_main, _xmlstream_error, evt);
}


int xmlstream_write(thread t, xmlnode x)
{
    tstream ts = (tstream)(t->data);
    tstream_buffer b;

    log_debug(ZONE,"xmlstream_write");

    if(t == NULL || x == NULL)
        return 0;

    b = pmalloc(xmlnode_pool(x),sizeof(struct tstream_buffer_struct));
    b->data = xmlnode2str(x);
    b->data_sz = strlen(b->data);
    b->sent = 0;
    b->obj = (void *)x;
    b->p = xmlnode_pool(x);
 
    if(!ts->flag1)
        tstream_write_buffer(t, b);
    else
        tstream_oob_buffer(t, b);

    return 0;
}

/* give a standard template xmlnode to work from */
xmlnode xmlstream_header(char *namespace, char *to, char *from)
{
    xmlnode x;
    char id[10];

    sprintf(id,"%X",time(NULL));

    x = xmlnode_new_tag("stream:stream");
    xmlnode_put_attrib(x, "xmlns:stream", "http://etherx.jabber.org/streams");
    xmlnode_put_attrib(x, "id", id);
    if(namespace != NULL)
        xmlnode_put_attrib(x, "xmlns", namespace);
    if(to != NULL)
        xmlnode_put_attrib(x, "to", to);
    if(from != NULL)
        xmlnode_put_attrib(x, "from", from);

    return x;
}

/* special wrapper to only write the root opening */
void xmlstream_write_header(thread t, xmlnode x)
{
    tstream ts = (tstream)(t->data);
    spool s;
    char *fixr, *head;

    if(xmlnode_has_children(x))
        return;

    s = spool_new(x->p);
    spooler(s,"<?xml version='1.0'?>",xmlnode2str(x),s);
    head = spool_print(s);
    fixr = strstr(head,"/>");
    *fixr = '>';
    ++fixr;
    *fixr = '\0';
    log_debug(ZONE,"head: %s",head);

    tstream_write(t, head, -1);
    if(ts->port > 0)
        ts->flag1 = 1; /* this is a connecting xmlstream, so flag to hold outgoing writes until we get back a valid xmlstream root */
    xmlnode_free(x);
}

/* special wrapper to send an error and close this stream */
void xmlstream_close(thread t)
{
    tstream_write(t, "</stream:stream>", 16);
    tstream_close(t);
}

/* special wrapper to send an error and close this stream */
void xmlstream_error(thread t, char *err)
{
    tstream_write(t, "<stream:error>", 14);
    tstream_write(t, err, -1);
    tstream_write(t, "</stream:error>",15);
    xmlstream_close(t);
}

void _xmlstream_startElement(void* userdata, const char* name, const char** atts)
{
    thread t = (thread)userdata;
    tstream ts = (tstream)(t->data);
    xmlnode current = (xmlnode)(ts->data2);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->data);

    if(current == NULL)
    {
        current = xmlnode_new_tag(name);
        xmlnode_put_expat_attribs(current, atts);

        if(ts->flag == 0)
        { /* this is a special flag saying that this is the <stream> root */
            ts->flag = 1;
            ts->data2 = current; /* so that the cleanup can free it if we exit in the handler */
            (onNode)(t, current, XMLSTREAM_ROOT);
            ts->flag1 = 0; /* we got the root, start sending data again */
            tstream_oob_flush(t);
            xmlnode_free(current);
            current = NULL;
        }
    }
    else
    {
        current = xmlnode_insert_tag(current, name);
        xmlnode_put_expat_attribs(current, atts);
    }

    ts->flag++;
    if(ts->flag > XML_MAX_DEPTH)
        xmlstream_error(t, "Maximum XML Element Depth Reached");

    ts->data2 = current;
}


void _xmlstream_endElement(void* userdata, const char* name)
{
    thread t = (thread)userdata;
    tstream ts = (tstream)(t->data);
    xmlnode current = (xmlnode)(ts->data2);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->data);
    xmlnode parent;

    /* if it's already NULL we've received </stream>, tell the app and we're outta here */
    if(current == NULL)
    {
        (onNode)(t, current, XMLSTREAM_CLOSE);
        pth_exit(NULL);
    }else{
        parent = xmlnode_get_parent(current);
        if(parent == NULL)
        { /* we are the top-most node, feed to the app and delete */
            (onNode)(t, current, XMLSTREAM_NODE);
            xmlnode_free(current);
        }
        ts->data2 = parent;
    }
    ts->flag--;
}


void _xmlstream_charData(void* userdata, const char* s, int len)
{
    thread t = (thread)userdata;
    tstream ts = (tstream)(t->data);
    xmlnode current = (xmlnode)(ts->data2);

    if(current == NULL)
    {
        /* we must be in the root of the stream where CDATA is irrelevant */
        return;
    }

    xmlnode_insert_cdata(current, s, len);
}


void _xmlstream_main(thread t, void *buffer, int bufsz)
{
    /*
    this is called when there is new data on the tstream
    it should process the XML using the special stream expat callbacks which
    call the onNode handler
    */

    tstream ts = (tstream)(t->data);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->data);
    XML_Parser p = (XML_Parser)(ts->data1);
    xmlnode n;

    log_debug(ZONE,"_xmlstream_main");

    if(buffer == NULL)
    { /* init */
        pth_cleanup_push(_xmlstream_cleanup, (void *)t);
        p = XML_ParserCreate(NULL);
        ts->data1 = (void *)p;
        ts->data2 = NULL;
        XML_SetUserData(p, (void *)t);
        XML_SetElementHandler(p, (void *)_xmlstream_startElement, (void *)_xmlstream_endElement);
        XML_SetCharacterDataHandler(p, (void *)_xmlstream_charData);
        (onNode)(t, NULL, XMLSTREAM_INIT);
        return;
    }

    if(!XML_Parse(p, buffer, bufsz, 0))
    {
        log_error("xmlstream","xmlstream_parse_error '%s' on socket '%s'",(char *)XML_ErrorString(XML_GetErrorCode(p)),ts->host);
        (onNode)(t, NULL, XMLSTREAM_ERROR);
        pth_exit(NULL);
    }

    n = (xmlnode)(ts->data2);
    if(pool_size(xmlnode_pool(n)) > 500000)
    {
        log_error("xmlstream","xml node size overrun on socket '%s'",ts->host);
        (onNode)(t, NULL, XMLSTREAM_ERROR);
        pth_exit(NULL);
    }
}

void _xmlstream_error(thread t, tstream_buffer buffer)
{
    /* the stream is closed and this was the data waiting to be written yet, feed it back to the app so it's not lost */
    tstream ts = (tstream)(t->data);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->data);
    xmlnode x = (xmlnode)(buffer->obj);

    log_debug(ZONE,"_xmlstream_error");

    if(buffer != NULL && x != NULL && onNode != NULL)
        (onNode)(t, x, XMLSTREAM_BOUNCE);
}

void _xmlstream_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    XML_Parser p = (XML_Parser)(ts->data1);
    xmlnode current = (xmlnode)(ts->data2);

    log_debug(ZONE,"_xmlstream_cleanup");
    if(current != NULL)
        xmlnode_free(current); /* cleanup anything left over */
    XML_ParserFree(p);
}
